//=============================================================================
//  TMidiIO class definition
//=============================================================================
// Author  : Adrian Meyer
//           based on Craig Stuart Sapp, craig@ccrma.stanford.edu
//           sequencer class
// Version : 0.2.0
// Log     :
//=============================================================================
// 04.03.2001 building class
//=============================================================================
// to do:
//=============================================================================

#ifndef __MCIO__
#define __MCIO__

#include <mcBase.h>
#include <strings.h>

//const long cMMDeviceCreate           = cMCDevice << 16 | 0x0001;

#define MIDI_EXTERNAL  (1)
#define MIDI_INTERNAL  (2)

typedef unsigned char uchar;

class TMidiIO: public TMidiBase
  {
    private:
      int fInputDevice;                               // active input device
      int fOutputDevice;                              // active output device

      void          buildInfoDatabase     (void);
      int           getFd                 (void) const;
      int           getInDeviceValue      (int aDevice) const;
      int           getInputType          (int aDevice) const;
      int           getOutDeviceValue     (int aDevice) const;
      int           getOutputType         (int aDevice) const;
      void          removeInfoDatabase    (void);
      void          setFd                 (int anFd);

      void          writeInternal(int aDevice, int aByte);
      void          transmitMessageToInternalSynth(void);
      void          transmitVoiceMessage(void);
      void          transmitCommonMessage(void);

   protected:
      static const char* sequencer;         // name of sequencer device
      static int    sequencer_fd;           // sequencer file descriptor
      static int    class_count;            // number of existing classes using
      static uchar  midi_write_packet[4];   // for writing MIDI bytes out
      static uchar  midi_read_packet[4];    // for reading MIDI bytes out
      static uchar  synth_write_message[8]; // for writing to internal seq
      static int    indevcount;             // number of MIDI input devices
      static int    outdevcount;            // number of MIDI output devices
      static char** indevnames;             // MIDI input device names
      static char** outdevnames;            // MIDI output device names
      static int*   indevnum;               // total number of MIDI inputs
      static int*   outdevnum;              // total number of MIDI outputs
      static int*   indevtype;              // 1 = External, 2 = Internal
      static int*   outdevtype;             // 1 = External, 2 = Internal
      static uchar  synth_message_buffer[1024];   // hold bytes for synth dev
      static int    synth_message_buffer_count;   // count of synth buffer
      static int    synth_message_bytes_expected; // expected count of synth
      static int    synth_message_curr_device;    // for keeping track of dev

    public:
      TMidiIO( int autoOpen = 1 );
      ~TMidiIO();

      void          close                (void);
      int GetInputCount( void ) const;
      int GetOutputCount( void ) const;
      const char* GetInputName ( int aDevice ) const;
      const char* GetOutputName ( int aDevice ) const;

      int           is_open              (void) const;
      int           open                 (void);
      void          read                 (uchar* buf, uchar* dev, int count);
      void          rawread              (uchar* buf, int packetCount);
      void          rebuildInfoDatabase  (void);
      void          Write                (int aDevice, int aByte);
      void          Write                (int aDevice, uchar* bytes, int count);
      void          Write                (int aDevice, int* bytes, int count);

      int GetInputDevice( void );                     // read input device
			void SetInputDevice( int aInputDevice );        // set input device
      int GetOutputDevice( void );                    // read output device
			void SetOutputDevice( int aOutputDevice );      // set output device
  };

#endif
