//=============================================================================
//  TMidiPlayback Implementation
//=============================================================================
#include <mcPlayback.h>
#include <gtk/gtk.h>

bool PlayTimerCallback( gpointer data );
bool GuiTimerCallback( gpointer data );

//=============================================================================
// constructor TMidiPlayback
// 05.03.2001 Adrian Meyer
//=============================================================================
TMidiPlayback::TMidiPlayback( TMidiSequence* aSequence, TMidiIO* aIO )
{
  fSequence = aSequence;
  fIO = aIO;
  fEventList = 0;
  fTempoList = 0;
  fPlayTimerID = -1;
  fGuiTimerID = -1;
}

//=============================================================================
// destructor TMidiPlayback
// 05.03.2001 Adrian Meyer
//=============================================================================
TMidiPlayback::~TMidiPlayback( void )
{
  Stop();
}

//=============================================================================
// playback timer
// 05.03.2001 Adrian Meyer
//=============================================================================
void TMidiPlayback::PlayTimer( void )
{
  double tmpInterval = 10.0;                          // timer interval 10 mili
  double tmpQuarterPerInterval = (( 60000000.0 / fTempoList->Tempo )/(60*1000)) * tmpInterval;
  double tmpTicksPerInterval = tmpQuarterPerInterval * fSequence->GetDeltaTimesType();
  tmpTicksPerInterval += fPlayRest;
  fPlayRest = tmpTicksPerInterval - (long)tmpTicksPerInterval;

  fPlayPosition+=(long)tmpTicksPerInterval;

  for ( i=0; i<fTrackCount; i++ )                     // check playable events
    if ( fEventList[i] )
    {                                                 // check offset
      while ( fEventList[i]->GetOffset() < fPlayPosition )
      {
        PlayEvent( fEventList[i] );                   // play the event
				fEventList[i] = fEventList[i]->Next;          // switch to next
        if ( !fEventList[i] ) break;                  // last event
      }
    }

  while ( fTempoList->Offset < fPlayPosition )        // check tempo
  {
    if ( fTempoList->Next )
      fTempoList = fTempoList->Next;
    else
      break;
  }


  if ( fPlayPosition > fEndPosition )                 // end reached
  {
    fPlayPosition = fEndPosition;                     // reset to end position
    Stop();                                           // stop playback
  }
}

//=============================================================================
// play midi event
// 05.03.2001 Adrian Meyer
//=============================================================================
void TMidiPlayback::PlayEvent( TMidiEvent* aEvent )
{
  aEvent->ShowInfo();
  fIO->Write( 0, aEvent->GetData1() );
  fIO->Write( 0, aEvent->GetData2() );
  fIO->Write( 0, aEvent->GetData3() );
}


//=============================================================================
// gui timer
// 05.03.2001 Adrian Meyer
//=============================================================================
void TMidiPlayback::GuiTimer( void )
{
  fSequence->SetPosition( fPlayPosition );            // update TMidiSequence
}

//=============================================================================
// initialize start
// 05.03.2001 Adrian Meyer
//=============================================================================
void TMidiPlayback::Start( void )
{
  if ( fPlayTimerID != -1 ) return;                   // already running

  fTempoList = fSequence->GetTempo(0);
  fTrackCount = fSequence->GetTrackCount();
                                                      // list with playable events
  fEventList = new TMidiEvent*[ fSequence->GetTrackCount() ];
  for ( i = 0; i<fTrackCount; i++ )
  {
    fEventList[i] = fSequence->GetTrack(i)->GetEvent(0);

    if ( fSequence->GetTrack(i)->GetInstrumentPatch() >= 0 )     // send initial program changes
    {
      fIO->Write( 0, cESProgramChange | ( fSequence->GetTrack(i)->GetChannel() - 1 ));
      fIO->Write( 0, fSequence->GetTrack(i)->GetInstrumentPatch());
    }
  }

  fPlayPosition = fSequence->GetPosition();           // grab current position
  fEndPosition = fSequence->GetMaxOffset();           // grab max offset
  fPlayRest = 0;
                                                      // reset to 0 on end
  if ( fPlayPosition == fEndPosition ) fPlayPosition = 0;

  fPlayTimerID = gtk_timeout_add( 10, (GtkFunction)PlayTimerCallback, this );
  fGuiTimerID = gtk_timeout_add( 80, (GtkFunction)GuiTimerCallback, this );
  CallMCMessage( this, cMCPlayback, cMMPlaybackStart, "playback started");
}

//=============================================================================
// stop playback
// 05.03.2001 Adrian Meyer
//=============================================================================
void TMidiPlayback::Stop( void )
{
  if ( fPlayTimerID == -1 ) return;                   // not running

  gtk_timeout_remove( fPlayTimerID );
  gtk_timeout_remove( fGuiTimerID );
  fPlayTimerID = -1;
  fGuiTimerID = -1;
  delete fEventList;

  fSequence->SetPosition( fPlayPosition );
  CallMCMessage( this, cMCPlayback, cMMPlaybackStop, "playback stoped");
}

//=============================================================================
// playback timer called
// 05.03.2001 Adrian Meyer
//=============================================================================
bool PlayTimerCallback(gpointer data)
{
  TMidiPlayback* playback = ((TMidiPlayback*)data );
  playback->PlayTimer();
  return true;
}

//=============================================================================
// update gui during playback
// 05.03.2001 Adrian Meyer
//=============================================================================
bool GuiTimerCallback(gpointer data)
{
  TMidiPlayback* playback = ((TMidiPlayback*)data );
  playback->GuiTimer();
  return true;
}
