//=============================================================================
//  TMidiTrack Class Implementation
//=============================================================================
#include <iostream.h>
#include <mcTrack.h>
//#define DEBUG_OUTPUT

//=============================================================================
// constructor TMidiTrack
// 21.02.2000 Adrian Meyer
//=============================================================================
TMidiTrack::TMidiTrack( TMidiSequence* aOwner )
{
	fOwner = aOwner;
  fTrackName = new char[1];
	fTrackName[0] = 0;
  fInstrumentName = new char[1];
	fInstrumentName[0] = 0;
	fLyricList = 0;
	fTextList = 0;
	fEventList = 0;
	fChannelPrefixList = 0;
	fEventCount = 0;
	fChannel = 1;
	fSelected = false;
  fInstrumentPatch = -1;

//  GetOwner()->CallMCMessage( this, cMCTrack, cMMTrackCreate, "track created");
}

//=============================================================================
// destructor TMidiTrack
// 28.08.2000 Adrian Meyer
//=============================================================================
TMidiTrack::~TMidiTrack( void )
{
	for ( int i = GetLyricCount()-1; i>=0 ; i-- ) { DeleteLyric( i ); }
	for ( int i = GetTextCount()-1; i>=0 ; i-- ) { DeleteText( i ); }
	for ( int i = GetChannelPrefixCount()-1; i>=0 ; i-- ) { DeleteChannelPrefix( i ); }

	TMidiEvent* tmpEvent = fEventList;									// delete events
	while ( tmpEvent != 0 )
	{
		TMidiEvent * oldEvent = tmpEvent;
		tmpEvent = tmpEvent->Next;
		oldEvent->Ref = 0;
		delete oldEvent;
	}	

//  GetOwner()->CallMCMessage( this, cMCTrack, cMMTrackDestroy, "track destroyed");
}

//=============================================================================
// read selected flag																	28.03.2000 Adrian Meyer
//=============================================================================
bool TMidiTrack::GetSelected( void )
{
	return fSelected;
}

//=============================================================================
// write selected flag                                28.03.2000 Adrian Meyer
//=============================================================================
void TMidiTrack::SetSelected( bool aSelected )
{
	fSelected = aSelected;
  GetOwner()->CallMCMessage( this, cMCTrack, cMMTrackSelect, "track selected");
}

//=============================================================================
// add event
// 25.02.2000 Adrian Meyer
//=============================================================================
TMidiEvent* TMidiTrack::AddEvent( long aOffset )
{
	TMidiEvent* tmpEvent;
	fEventCount++;

	if ( fEventList == 0 )															// empty list
 	{
		fEventList = new TMidiEvent( this, aOffset );		
		return fEventList;
	}
  else
	{
		if ( aOffset < fEventList->GetOffset() ) 						// first position
		{
			fEventList->Prev = new TMidiEvent( this, aOffset );
			fEventList->Prev->Next = fEventList;
			fEventList = fEventList->Prev;			
			return fEventList;
		}
		else
		{
    	tmpEvent = fEventList;														// find offset position
    	while ( tmpEvent->GetOffset() <= aOffset && tmpEvent->Next != 0 )
    	{
    		tmpEvent = tmpEvent->Next;
    	}

    	TMidiEvent* oldNext = tmpEvent->Next;
    	tmpEvent->Next = new TMidiEvent( this, aOffset );
    	tmpEvent->Next->Prev = tmpEvent;
    	tmpEvent->Next->Next = oldNext;

    	if ( oldNext != 0 )																// not last event
    	{
    		oldNext->Prev = tmpEvent->Next;
    	}
			return tmpEvent->Next;
		}
	}

	return 0;
}

//=============================================================================
// set new offset
// 25.02.2000 Adrian Meyer
//=============================================================================
void TMidiTrack::SetEventOffset( long aID, long aOffset )
{
	TMidiEvent* tmpEvent = GetEvent( aID );
	
	cout << "Modify " << tmpEvent->GetOffset() << " to " << aOffset << endl;

	if ( aID > fEventCount - 1 ) { return; }

	TMidiEvent* posEvent = fEventList;

	if ( fEventList != 0 )	
	{
		while ( posEvent->Next != 0  )
		{	
			if ( posEvent->Next->GetOffset() < aOffset )
				{	posEvent = posEvent->Next;}
			else
				{ break; }
		}
	}	

	if ( posEvent != tmpEvent )                             // same position?
	{                                                       // NO!
  																												// unchain
  	if ( tmpEvent->Next != 0 ) { tmpEvent->Next->Prev = tmpEvent->Prev; }
  	if ( tmpEvent->Prev != 0 ) { tmpEvent->Prev->Next = tmpEvent->Next; }
  	if ( tmpEvent == fEventList ) { fEventList = tmpEvent->Next; }

		tmpEvent->Next = posEvent->Next;
		posEvent->Next = tmpEvent;
		tmpEvent->Prev = posEvent;
		if ( tmpEvent->Next != 0 )
		{
			tmpEvent->Next->Prev = tmpEvent;
		}
	  GetOwner()->CallMCMessage( this, cMCTrack, cMMTrackEventListChanged, "event list changed");
	}

	tmpEvent->SetOffset( aOffset );
}

//=============================================================================
// read event
// 25.02.2000 Adrian Meyer
//=============================================================================
TMidiEvent* TMidiTrack::GetEvent( long aID )
{
 	TMidiEvent* tmpEvent = fEventList;
 	for ( long i=0; i<aID; i++ )
 	{
 		tmpEvent = tmpEvent->Next;
 	}
 	return tmpEvent;
}

//=============================================================================
// read event counter
// 25.02.2000 Adrian Meyer
//=============================================================================
long TMidiTrack::GetEventCount( void )
{
	return fEventCount;
}

//=============================================================================
// adjust count
// 25.02.2000 Adrian Meyer
//=============================================================================
void TMidiTrack::SetEventCount( long aCount )
{
	fEventCount = aCount;
}

//=============================================================================
// delete event
// 25.02.2000 Adrian Meyer
//=============================================================================
void TMidiTrack::DeleteEvent( long aID )
{
	TMidiEvent* oldEvent = GetEvent( aID );
	TMidiEvent* refEvent = 0;
	fEventCount--;

	if ( GetEvent( aID )->Ref != 0 )										// referenced event (note off)
	{
		fEventCount--;
		refEvent = GetEvent( aID )->Ref;
	}

	if ( aID == 0 )																			// first event?
	{
		if ( fEventList != 0 )														// correct starting point of chain
		{
			fEventList = fEventList->Next;
			if ( fEventList != 0 )
				{	fEventList->Prev = 0; }
		}
	}
	else
	{
		oldEvent->Prev->Next = oldEvent->Next;						// unchain event
		if ( oldEvent->Next != 0 )
			{ oldEvent->Next->Prev = oldEvent->Prev;	}
	}

	if ( refEvent	!= 0 )																// unchain referenced event
	{
		if ( refEvent->Prev == 0 )    										// correct starting point of chain
		{
			fEventList = refEvent->Next;
			if ( fEventList != 0 )
				{	fEventList->Prev = 0; }
		}
		else
		{
			if ( refEvent->Prev != 0 )
				{	refEvent->Prev->Next = refEvent->Next; };
  		if ( refEvent->Next != 0 )
  			{ refEvent->Next->Prev = refEvent->Prev; };
		}
		delete refEvent;																	// delete referenced event
	}

	delete oldEvent;																		// delete event
}

//=============================================================================
// write track name
// 20.02.2000 Adrian Meyer
//=============================================================================
void TMidiTrack::SetTrackName( char *aTrackName )
{
  if ( fTrackName != 0 )
  {
    delete fTrackName;
  }
  fTrackName = new char[ strlen( aTrackName ) + 1 ];
  strcpy( fTrackName, aTrackName );
//	fTrackName[ strlen( aTrackName ) ] = 0;

  GetOwner()->CallMCMessage( this, cMCTrack, cMMTrackSetName, "track name set");
	GetOwner()->SetChanged( true );	
}

//=============================================================================
// read track name
// 20.02.2000 Adrian Meyer
//=============================================================================
char* TMidiTrack::GetTrackName( void )
{
  return fTrackName;
}

//=============================================================================
// write instrument name
// 20.02.2000 Adrian Meyer
//=============================================================================
void TMidiTrack::SetInstrumentName( char *aInstrumentName )
{
  if ( fInstrumentName != 0 )
  {
    delete fInstrumentName;
  }
  fInstrumentName = new char[ strlen( aInstrumentName ) + 1 ];
  strcpy( fInstrumentName, aInstrumentName );

  GetOwner()->CallMCMessage( this, cMCTrack, cMMTrackSetInstrumentName, "instrument name set");
}

//=============================================================================
// read instrument name
// 20.02.2000 Adrian Meyer
//=============================================================================
char* TMidiTrack::GetInstrumentName( void )
{
  return fInstrumentName;
}

//=============================================================================
// write channel																			27.03.2000 Adrian Meyer
//=============================================================================
void TMidiTrack::SetChannel( short aChannel )
{
	fChannel = aChannel;

  GetOwner()->CallMCMessage( this, cMCTrack, cMMTrackSetChannel, "track channel set");
}

//=============================================================================
// read channel																				27.03.2000 Adrian Meyer
//=============================================================================
short TMidiTrack::GetChannel( void )
{
	return fChannel;
}


//=============================================================================
// add Lyric
// 23.02.2000 Adrian Meyer
//=============================================================================
void TMidiTrack::AddLyric( char* aLyric, long aOffset )
{
  long tmpID = 0;
	sLyricList* tmpLyricList;

  if ( fLyricList == 0 )
  {
    tmpLyricList = fLyricList = new sLyricList;
  }
  else
  {
    tmpLyricList = fLyricList;
		while ( tmpLyricList->Next != 0 )
    {
			tmpLyricList = tmpLyricList->Next;
    }
    tmpLyricList->Next = new sLyricList;
	  tmpID = tmpLyricList->ID;
		tmpLyricList = tmpLyricList->Next;
    tmpID++;
  }

  tmpLyricList->Next = 0;
  tmpLyricList->ID = tmpID;
  tmpLyricList->Offset = aOffset;
  tmpLyricList->Lyric = new char[ sizeof( aLyric ) + 1 ];
  strcpy( tmpLyricList->Lyric, aLyric );

  GetOwner()->CallMCMessage( this, cMCTrack, cMMTrackAddLyric, "lyric added");
}

//=============================================================================
// rename Lyric
// 23.02.2000 Adrian Meyer
//=============================================================================
void TMidiTrack::RenameLyric( char* aLyric, long aID )
{
	if ( GetLyric( aID ) != 0 )
	{
		delete GetLyric( aID )->Lyric;
		GetLyric( aID )->Lyric = new char[ sizeof( aLyric ) + 1 ];
    strcpy( GetLyric( aID )->Lyric, aLyric );
	} 	
  GetOwner()->CallMCMessage( this, cMCTrack, cMMTrackRenameLyric, "lyric renamed");
}

//=============================================================================
// delete a Lyric
// 23.02.2000 Adrian Meyer
//=============================================================================
void TMidiTrack::DeleteLyric( long aID )
{
	sLyricList* tmpLyricList = 0;
	sLyricList* delLyricList = GetLyric( aID );

  if ( delLyricList != 0 )														// delete the Lyric
  {
    if ( fLyricList == delLyricList )								// deleting 1st
    {
      fLyricList = delLyricList->Next;
    }
		else
		{
			GetLyric( aID - 1 )->Next = delLyricList->Next;			
		}

    if ( delLyricList->Lyric != 0 ) 									// delete name
    {
			delete delLyricList->Lyric;
    }
    delete delLyricList;
  }

  if ( fLyricList != 0 )															// renumber ID's
  {
		tmpLyricList = fLyricList;
    long tmpID = 0;

    while ( tmpLyricList != 0 )
    {
      tmpLyricList->ID = tmpID++;
			tmpLyricList = tmpLyricList->Next;
    }
  }
  GetOwner()->CallMCMessage( this, cMCTrack, cMMTrackDeleteLyric, "lyric deleted");
}

//=============================================================================
// read Lyric list
// 23.02.2000 Adrian Meyer
//=============================================================================
sLyricList* TMidiTrack::GetLyric( long aID )
{
  sLyricList* tmpLyricList = fLyricList;
  if ( fLyricList != 0 )
  {
    while (( tmpLyricList->Next != 0 ) && ( tmpLyricList->ID != aID ))
    {
      tmpLyricList = tmpLyricList->Next;
    }
    return tmpLyricList;
  }
  return fLyricList;
}

//=============================================================================
// read Lyric list count
// 23.02.2000 Adrian Meyer
//=============================================================================
long TMidiTrack::GetLyricCount( void )
{
  sLyricList* tmpLyricList = fLyricList;
  if ( fLyricList != 0 )
  {
    while ( tmpLyricList->Next != 0 )
    {
      tmpLyricList = tmpLyricList->Next;
    }
    return tmpLyricList->ID + 1;
  }
  else
  {
	  return 0;
  }
}

//=============================================================================
// add Text
// 23.02.2000 Adrian Meyer
//=============================================================================
void TMidiTrack::AddText( char* aText, long aOffset )
{
  long tmpID = 0;
	sTextList* tmpTextList;

  if ( fTextList == 0 )
  {
    tmpTextList = fTextList = new sTextList;
  }
  else
  {
    tmpTextList = fTextList;
		while ( tmpTextList->Next != 0 )
    {
			tmpTextList = tmpTextList->Next;
    }
    tmpTextList->Next = new sTextList;
	  tmpID = tmpTextList->ID;
		tmpTextList = tmpTextList->Next;
    tmpID++;
  }

  tmpTextList->Next = 0;
  tmpTextList->ID = tmpID;
  tmpTextList->Offset = aOffset;
  tmpTextList->Text = new char[ sizeof( aText ) + 1 ];
  strcpy( tmpTextList->Text, aText );

  GetOwner()->CallMCMessage( this, cMCTrack, cMMTrackAddText, "text added");
}

//=============================================================================
// rename Text
// 23.02.2000 Adrian Meyer
//=============================================================================
void TMidiTrack::RenameText( char* aText, long aID )
{
	if ( GetText( aID ) != 0 )
	{
		delete GetText( aID )->Text;
		GetText( aID )->Text = new char[ sizeof( aText ) + 1 ];
    strcpy( GetText( aID )->Text, aText );
	} 	
  GetOwner()->CallMCMessage( this, cMCTrack, cMMTrackRenameText, "text renamed");
}

//=============================================================================
// delete a Text
// 23.02.2000 Adrian Meyer
//=============================================================================
void TMidiTrack::DeleteText( long aID )
{
	sTextList* tmpTextList = 0;
	sTextList* delTextList = GetText( aID );

  if ( delTextList != 0 )														// delete the Text
  {
    if ( fTextList == delTextList )								// deleting 1st
    {
      fTextList = delTextList->Next;
    }
		else
		{
			GetText( aID - 1 )->Next = delTextList->Next;			
		}

    if ( delTextList->Text != 0 ) 									// delete name
    {
			delete delTextList->Text;
    }
    delete delTextList;
  }

  if ( fTextList != 0 )															// renumber ID's
  {
		tmpTextList = fTextList;
    long tmpID = 0;

    while ( tmpTextList != 0 )
    {
      tmpTextList->ID = tmpID++;
			tmpTextList = tmpTextList->Next;
    }
  }
  GetOwner()->CallMCMessage( this, cMCTrack, cMMTrackDeleteText, "text deleted");
}

//=============================================================================
// read Text list
// 23.02.2000 Adrian Meyer
//=============================================================================
sTextList* TMidiTrack::GetText( long aID )
{
  sTextList* tmpTextList = fTextList;
  if ( fTextList != 0 )
  {
    while (( tmpTextList->Next != 0 ) && ( tmpTextList->ID != aID ))
    {
      tmpTextList = tmpTextList->Next;
    }
    return tmpTextList;
  }
  return fTextList;
}

//=============================================================================
// read Text list count
// 23.02.2000 Adrian Meyer
//=============================================================================
long TMidiTrack::GetTextCount( void )
{
  sTextList* tmpTextList = fTextList;
  if ( fTextList != 0 )
  {
    while ( tmpTextList->Next != 0 )
    {
      tmpTextList = tmpTextList->Next;
    }
    return tmpTextList->ID + 1;
  }
  else
  {
	  return 0;
  }
}

//=============================================================================
// add channel prefix
// 23.02.2000 Adrian Meyer
//=============================================================================
void TMidiTrack::AddChannelPrefix( short aChannelPrefix, long aOffset )
{
  long tmpID = 0;
	sChannelPrefixList* tmpChannelPrefixList;

  if ( fChannelPrefixList == 0 )
  {
    tmpChannelPrefixList = fChannelPrefixList = new sChannelPrefixList;
  }
  else
  {
    tmpChannelPrefixList = fChannelPrefixList;
		while ( tmpChannelPrefixList->Next != 0 )
    {
			tmpChannelPrefixList = tmpChannelPrefixList->Next;
    }
    tmpChannelPrefixList->Next = new sChannelPrefixList;
	  tmpID = tmpChannelPrefixList->ID;
		tmpChannelPrefixList = tmpChannelPrefixList->Next;
    tmpID++;
  }

  tmpChannelPrefixList->Next = 0;
  tmpChannelPrefixList->ID = tmpID;
  tmpChannelPrefixList->Offset = aOffset;
  tmpChannelPrefixList->ChannelPrefix = aChannelPrefix;

  GetOwner()->CallMCMessage( this, cMCTrack, cMMTrackAddChannelPrefix, "channel prefix added");
}

//=============================================================================
// delete a channel prefix
// 23.02.2000 Adrian Meyer
//=============================================================================
void TMidiTrack::DeleteChannelPrefix( long aID )
{
	sChannelPrefixList* tmpChannelPrefixList = 0;
	sChannelPrefixList* delChannelPrefixList = GetChannelPrefix( aID );

  if ( delChannelPrefixList != 0 )										// delete the ChannelPrefix
  {
    if ( fChannelPrefixList == delChannelPrefixList )	// deleting 1st
    {
      fChannelPrefixList = delChannelPrefixList->Next;
    }
		else
		{
			GetChannelPrefix( aID - 1 )->Next = delChannelPrefixList->Next;			
		}

    delete delChannelPrefixList;
  }

  if ( fChannelPrefixList != 0 )										  // renumber ID's
  {
		tmpChannelPrefixList = fChannelPrefixList;
    long tmpID = 0;

    while ( tmpChannelPrefixList != 0 )
    {
      tmpChannelPrefixList->ID = tmpID++;
			tmpChannelPrefixList = tmpChannelPrefixList->Next;
    }
  }
  GetOwner()->CallMCMessage( this, cMCTrack, cMMTrackDeleteChannelPrefix, "channel prefix deleted");
}

//=============================================================================
// read channel prefix list
// 23.02.2000 Adrian Meyer
//=============================================================================
sChannelPrefixList* TMidiTrack::GetChannelPrefix( long aID )
{
  sChannelPrefixList* tmpChannelPrefixList = fChannelPrefixList;
  if ( fChannelPrefixList != 0 )
  {
    while (( tmpChannelPrefixList->Next != 0 ) && ( tmpChannelPrefixList->ID != aID ))
    {
      tmpChannelPrefixList = tmpChannelPrefixList->Next;
    }
    return tmpChannelPrefixList;
  }
  return fChannelPrefixList;
}

//=============================================================================
// read channel prefix list count
// 23.02.2000 Adrian Meyer
//=============================================================================
long TMidiTrack::GetChannelPrefixCount( void )
{
  sChannelPrefixList* tmpChannelPrefixList = fChannelPrefixList;
  if ( fChannelPrefixList != 0 )
  {
    while ( tmpChannelPrefixList->Next != 0 )
    {
      tmpChannelPrefixList = tmpChannelPrefixList->Next;
    }
    return tmpChannelPrefixList->ID + 1;
  }
  else
  {
	  return 0;
  }
}

//=============================================================================
// read the tracks index															05.04.2000 Adrian Meyer
//=============================================================================
int TMidiTrack::GetIndex( void )
{
	for ( long i = 0; i<fOwner->GetTrackCount(); i++ )
	{
		if ( this == fOwner->GetTrack(i) ) { return i; }		
	}
	return fOwner->GetTrackCount() - 1;
}

//=============================================================================
// read owner																					06.04.2000 Adrian Meyer
//=============================================================================
TMidiSequence* TMidiTrack::GetOwner( void )
{
	return fOwner;
}

//=============================================================================
// set instrument patch #
// 03.09.2000 Adrian Meyer
//=============================================================================
void TMidiTrack::SetInstrumentPatch( short aInstrumentPatch )
{
	fInstrumentPatch = aInstrumentPatch;
  GetOwner()->CallMCMessage( this, cMCTrack, cMMTrackSetIntrumentPatch, "instrument patch set");
}

//=============================================================================
// read instrument patch
// 03.09.2000 Adrian Meyer
//=============================================================================
short TMidiTrack::GetInstrumentPatch( void )
{
	return fInstrumentPatch;
}