//=============================================================================
//  model view control marker properties
//=============================================================================
#include <mvcMarkerProperties.h>

// callbacks
void OkClickedMarkerProp( GtkButton *button, gpointer user_data );
void CancelClickedMarkerProp( GtkButton *button, gpointer user_data );
void MeasureChangedMarkerProp( GtkEditable *editable, gpointer user_data );

//=============================================================================
// construtor MvcMarkerProperties											
// 11.09.2000 Adrian Meyer
//=============================================================================
TMvcMarkerProperties::TMvcMarkerProperties( TMidiSequence* aSequence, long aMarkerID )
{
  wndMarkerProperties = create_wndMarkerProperties();	// call glade
	AddChildrenToWidgetList( wndMarkerProperties );     // fetch widgets
	fSequence = aSequence;
	fMarkerID = aMarkerID;
	
  HookSignal( "btnOk", "clicked", GTK_SIGNAL_FUNC( OkClickedMarkerProp ), this );
  HookSignal( "btnCancel", "clicked", GTK_SIGNAL_FUNC( CancelClickedMarkerProp ), this );
  HookSignal( "spiMeasure", "changed", GTK_SIGNAL_FUNC( MeasureChangedMarkerProp ), this );

/*
	char tmpStr[100];
	char* tmpItem = new char[ strlen("[none]")];				// no patch set

	fGMPatchNames = NULL;
	strcpy( tmpItem, "[none]" );
	fGMPatchNames = g_list_append( fGMPatchNames, tmpItem );

	for ( int i = 0; i<128; i++ )												// add only numbers
	{
		sprintf( tmpStr, "[%.3d] %s", i+1, cGMPatchName[ i ]);
//		strcat( tmpStr,  );
		char* tmpItem = new char[ strlen(tmpStr) + 1 ];
		strcpy( tmpItem, tmpStr );
		fGMPatchNames = g_list_append( fGMPatchNames, tmpItem );
	}

	gtk_combo_set_popdown_strings( GTK_COMBO( FindWidget( "cboInstrumentPatch" )), fGMPatchNames );	*/
	if ( aSequence == 0 ) return;
  gtk_widget_show( wndMarkerProperties );							

	SetData();																					// set data to gui
}

//=============================================================================
// destrutor MvcMain			
// 11.09.2000 Adrian Meyer
//=============================================================================
TMvcMarkerProperties::~TMvcMarkerProperties( void )
{
	delete wndMarkerProperties;
}

//=============================================================================
// set data to gui				
// 11.09.2000 Adrian Meyer
//=============================================================================
void TMvcMarkerProperties::SetData( void )
{
 	long tmpMeasure = 0; long tmpBeat = 0; long tmpTick = 0;		

 	fSequence->GetMeasureBeatTick( fSequence->GetMarker( fMarkerID )->Offset, &tmpMeasure, &tmpBeat, &tmpTick );

 																										// set measure
   gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiMeasure" )), tmpMeasure );
   gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiBeat" )), tmpBeat );
   gtk_spin_button_set_value( GTK_SPIN_BUTTON( FindWidget( "spiTick" )), tmpTick );

	gtk_entry_set_text( GTK_ENTRY( FindWidget( "edtMarkerName" )),
		fSequence->GetMarker( fMarkerID )->MarkerName );
}

//=============================================================================
// read data from gui into object											
// 11.09.2000 Adrian Meyer
//=============================================================================
void TMvcMarkerProperties::GetData( void )
{
	fSequence->RenameMarker( gtk_entry_get_text( GTK_ENTRY( FindWidget( "edtMarkerName" ))),
													 fMarkerID );								// rename marker

  long tmpOffset =																		// new offset
		fSequence->GetOffset(
			gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( FindWidget( "spiMeasure" ))),
			gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( FindWidget( "spiBeat" ))),
			gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( FindWidget( "spiTick" )))
    );

	fSequence->GetMarker( fMarkerID )->Offset = tmpOffset;
	fSequence->SortMarker();
}

//=============================================================================
// read widget						
// 11.09.2000 Adrian Meyer
//=============================================================================
GtkWidget* TMvcMarkerProperties::GetWidget( void )
{
	return wndMarkerProperties;
}

//=============================================================================
// adjust beat/ticks to selected measure
// 12.09.2000 Adrian Meyer
//=============================================================================
void TMvcMarkerProperties::AdjustToMeasure( long aMeasure )
{
	long tmpOffset = fSequence->GetOffset( aMeasure + 1, 0, 0 ) - 1;
	long tmpMeasure = 0;
	long tmpBeat = 0;
	long tmpTick = 0;

	fSequence->GetMeasureBeatTick( tmpOffset, &tmpMeasure, &tmpBeat, &tmpTick );

	gtk_spin_button_get_adjustment(	GTK_SPIN_BUTTON( FindWidget( "spiBeat" )))->upper = tmpBeat + 1;
	gtk_spin_button_get_adjustment(	GTK_SPIN_BUTTON( FindWidget( "spiTick" )))->upper = tmpTick;
}

//=============================================================================
// ok clicked																					
// 11.09.2000 Adrian Meyer
//=============================================================================
void OkClickedMarkerProp( GtkButton *button, gpointer user_data )
{
	gtk_widget_hide( ((TMvcMarkerProperties*)user_data)->GetWidget() );
	((TMvcMarkerProperties*)user_data)->GetData();
}

//=============================================================================
// cancel clicked																			
// 11.09.2000 Adrian Meyer
//=============================================================================
void CancelClickedMarkerProp( GtkButton *button, gpointer user_data )
{
	gtk_widget_hide( ((TMvcMarkerProperties*)user_data)->GetWidget() );
}

//=============================================================================
// measure changed
// 12.09.2000 Adrian Meyer
//============================================================================
void MeasureChangedMarkerProp( GtkEditable *editable, gpointer user_data )
{
	((TMvcMarkerProperties*)user_data)->AdjustToMeasure( gtk_spin_button_get_value_as_int ( GTK_SPIN_BUTTON( editable )));		
}
