//=============================================================================
//  model view control midi device settings
//=============================================================================
#include <mvcMidiDeviceOptions.h>
#include <stdio.h>

void OkClickedMidiDeviceOpt( GtkButton *button, gpointer user_data );
void CancelClickedMidiDeviceOpt( GtkButton *button, gpointer user_data );
void NoteButtonClickedMidiDeviceOpt( GtkButton *button, gpointer user_data );

//=============================================================================
// constructor
// 20.02.2001 Adrian Meyer
//=============================================================================
TMvcMidiDeviceOptions::TMvcMidiDeviceOptions( TMidiIO* aIO )
{
  wndMidiDeviceOptions = create_wndMidiDeviceOptions();
	AddChildrenToWidgetList( wndMidiDeviceOptions );   // fetch widgets

  HookSignal( "btnOk", "clicked", GTK_SIGNAL_FUNC( OkClickedMidiDeviceOpt ), this );
  HookSignal( "btnCancel", "clicked", GTK_SIGNAL_FUNC( CancelClickedMidiDeviceOpt ), this );

  fIO = aIO;									             						// init values
	char tmpStr[255];

	if ( fIO == 0 ) return;

  fInputDeviceNames = 0;    													// set input device list
	for ( int i = 0; i<fIO->GetInputCount(); i++ )				
	{
		sprintf( tmpStr, "[%.3d]  %s", i, fIO->GetInputName(i) );
		char* tmpItem = new char[ strlen( tmpStr ) +1 ];
		tmpItem[0] = 0;
		strcpy( tmpItem, tmpStr );
		fInputDeviceNames = g_list_append( fInputDeviceNames, tmpItem );
	}
  if ( fInputDeviceNames )
		gtk_combo_set_popdown_strings( GTK_COMBO( FindWidget( "cboInputDevice" )), fInputDeviceNames );	
	else
		gtk_widget_set_sensitive( FindWidget( "cboInputDevice" ), false );


  fOutputDeviceNames = 0;															// set output device list
	for ( int i = 0; i<fIO->GetOutputCount(); i++ )				
	{
		sprintf( tmpStr, "[%.3d]  %s", i, fIO->GetOutputName(i) );
		char* tmpItem = new char[ strlen(tmpStr) +1 ];
		tmpItem[0] = 0;
		strcpy( tmpItem, tmpStr );
		fOutputDeviceNames = g_list_append( fOutputDeviceNames, tmpItem );
	}
	if ( fOutputDeviceNames )
		gtk_combo_set_popdown_strings( GTK_COMBO( FindWidget( "cboOutputDevice" )), fOutputDeviceNames );	
	else
		gtk_widget_set_sensitive( FindWidget( "cboOutputDevice" ), false );

  gtk_widget_show( wndMidiDeviceOptions );							
  SetData();
}

//=============================================================================
// destructor
// 20.02.2001 Adrian Meyer
//=============================================================================
TMvcMidiDeviceOptions::~TMvcMidiDeviceOptions( )
{
	delete wndMidiDeviceOptions;                       // remove window
}

//=============================================================================
// read widget
// 20.02.2001 Adrian Meyer
//=============================================================================
GtkWidget* TMvcMidiDeviceOptions::GetWidget( void )
{
  return wndMidiDeviceOptions;                       // return widget
}

//=============================================================================
// set data to gui
// 21.02.2001 Adrian Meyer
//=============================================================================
void TMvcMidiDeviceOptions::SetData( void )
{
}

//=============================================================================
// read data from gui
// 21.02.2001 Adrian Meyer
//=============================================================================
void TMvcMidiDeviceOptions::GetData( void )
{
}

//=============================================================================
// OK clicked
// 20.02.2001 Adrian Meyer
//=============================================================================
void OkClickedMidiDeviceOpt( GtkButton *button, gpointer user_data )
{
  (( TMvcMidiDeviceOptions* )user_data)->GetData();
	gtk_widget_hide( ((TMvcMidiDeviceOptions*)user_data)->GetWidget() );
}

//=============================================================================
// cancel clicked
// 20.02.2001 Adrian Meyer
//=============================================================================
void CancelClickedMidiDeviceOpt( GtkButton *button, gpointer user_data )
{
	gtk_widget_hide( ((TMvcMidiDeviceOptions*)user_data)->GetWidget() );
}
