//=============================================================================
// model view control controller
//=============================================================================
#include <mvcController.h>

// callbacks
void ControllerTimeZoomChanged( GtkAdjustment *adjustment, gpointer user_data );
bool ControllerExposeHook( GtkWidget *widget, GdkEventExpose *event, gpointer user_data );
void ControllerScroll( GtkAdjustment *adjustment, gpointer user_data );


bool ControllerMouseDown( GtkWidget *widget, GdkEventButton *event, gpointer user_data);
bool ControllerMouseUp( GtkWidget *widget, GdkEventButton *event, gpointer user_data);
bool ControllerMouseMove( GtkWidget *widget, GdkEventMotion *event, gpointer user_data);

static short cXMaxMeasureCount = 20;
static short cXMinMeasureCount = 2;
static short cCountMeasureTail = 5;

//=============================================================================
// constructor																				
// 22.02.2001 Adrian Meyer
//=============================================================================
TMvcController::TMvcController( GtkWidget* aWidget, TMidiSequence* aSequence )
{
	fWidget = aWidget;
	AddChildrenToWidgetList( aWidget );                 // fetch widgets
	fSequence = aSequence;															// take sequence
	fWidth = 0;
	fHeight = 0;
	fTicksPerPix = 0;
	fSelectionActive = false;
	fWindowWidth = 0;
	fBufferMain = 0;
	fBufferLabel = 0;
	drwControllerMain =  FindWidget( "drwControllerMain" );

  fXAdjustment = gtk_range_get_adjustment( GTK_RANGE( FindWidget( "scrControllerMainX" )));

	fHeader = new TMvcHeader ( FindWidget( "drwControllerHeader" ), fSequence, fXAdjustment );

	gtk_signal_connect(	GTK_OBJECT( fXAdjustment ),	"value-changed",
				GTK_SIGNAL_FUNC( ControllerTimeZoomChanged ),	this );

  HookSignal( "drwControllerMain", "expose_event", GTK_SIGNAL_FUNC( ControllerExposeHook ), this );
  HookSignal( "drwControllerMain", "button_press_event", GTK_SIGNAL_FUNC( ControllerMouseDown ), this );
  HookSignal( "drwControllerMain", "button_release_event",GTK_SIGNAL_FUNC( ControllerMouseUp ), this );
  HookSignal( "drwControllerMain", "motion_notify_event", GTK_SIGNAL_FUNC( ControllerMouseMove ), this );	

                                                      // activate events
	gtk_widget_add_events(
		drwControllerMain,
		GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK );

	gtk_scrolled_window_set_hadjustment(								// synchronize main and header area
		GTK_SCROLLED_WINDOW( FindWidget( "scrControllerHeader" )), fXAdjustment );

	gtk_signal_connect(	GTK_OBJECT( fXAdjustment ),	"value-changed",
				GTK_SIGNAL_FUNC( ControllerScroll ),	this );

	colBackground = new GdkColor;
	colBackground->red = 65535;	colBackground->green = 65535;	colBackground->blue = 65535;
	gdk_colormap_alloc_color( gtk_widget_get_colormap( drwControllerMain),	
		colBackground, true, true );

	colGrid = new GdkColor;
	colGrid->red = 10000;	colGrid->green = 10000;	colGrid->blue = 10000;
	gdk_colormap_alloc_color( gtk_widget_get_colormap( drwControllerMain),	
		colGrid, true, true );

	colSubGrid = new GdkColor;
	colSubGrid->red = 55000;	colSubGrid->green = 55000;	colSubGrid->blue = 55000;
	gdk_colormap_alloc_color( gtk_widget_get_colormap( drwControllerMain),	
		colSubGrid, true, true );

	colMidi = new GdkColor;
	colMidi->red = 0;	colMidi->green = 0;	colMidi->blue = 32000;
	gdk_colormap_alloc_color( gtk_widget_get_colormap( drwControllerMain),	
		colMidi, true, true );

	colSelection = new GdkColor;
	colSelection->red = colBackground->red - 65535;
	colSelection->green = colBackground->green - 55000;
	colSelection->blue = colBackground->blue - 55000;
	gdk_colormap_alloc_color( gtk_widget_get_colormap( drwControllerMain),	
		colSelection, true, true );
}

//=============================================================================
// message from midi classes
// 22.02.2001 Adrian Meyer
//=============================================================================
void TMvcController::MCMessage( TMidiBase* aMidiBase, sMCMessage* aMCMessage )
{
	switch ( aMCMessage->Message )
	{
		case cMMSequenceAddTrack:
		case cMMSequenceOpenFile:
		case cMMSequenceNew:
				Resize();
			break;

		case cMMSequenceSetSelectionStart:
		case cMMSequenceSetSelectionEnd:
				gtk_widget_queue_draw( drwControllerMain );
			break;

	}

	if ( fHeader != 0 ) { fHeader->MCMessage( aMidiBase, aMCMessage );}
}

//=============================================================================
// read widget																			
// 22.02.2001 Adrian Meyer
//=============================================================================
GtkWidget* TMvcController::GetWidget( void )
{
	return fWidget;
}

//=============================================================================
// set new with and height
// 22.02.2001 Adrian Meyer
//=============================================================================
void TMvcController::Resize( void )
{
	if ( FindWidget( "vpoControllerMain" )->window == 0 ) {return;}

	long m; long b; long t;
	fSequence->GetMeasureBeatTick( fSequence->GetMaxOffset(), &m, &b, &t );
	long maxOffset = fSequence->GetOffset( m + 1, 1, 0 ) - 1 + fSequence->GetOffset( cCountMeasureTail + 1, 1, 0 ) - 1;
	long maxOffset20 = fSequence->GetOffset( cXMaxMeasureCount + 1, 1, 0 ) - 1;
	if ( maxOffset < maxOffset20 )											// set measure count
	{
		fMeasureCount = cXMaxMeasureCount;                // correct to max
	}
	else
	{
		long tmpBeat = 0;	long tmpTick = 0;								// more than max
		fSequence->GetMeasureBeatTick( fSequence->GetMaxOffset(),
			&fMeasureCount, &tmpBeat, &tmpTick );		
	}

	maxOffset = 0;
	for ( int i=1; i<=fMeasureCount; i++ )
	{
		long tmpBeatCount; long tmpTicksPerBeat;
  	fSequence->GetTimeInformation( i, &tmpBeatCount, &tmpTicksPerBeat );
		maxOffset += tmpBeatCount * tmpTicksPerBeat;
	}	

	long minOffset = fSequence->GetOffset( cXMinMeasureCount + 1, 1, 0 ) - 1;
  long tmpZoomFactorTime = (int)gtk_range_get_adjustment( GTK_RANGE( FindWidget(	"sclControllerTimeZoom" )))->value;

																											// width of viewable area
	gdk_window_get_size( FindWidget( "vpoControllerMain" )->window, &fWindowWidth, &fHeight );
	fWindowWidth -= 1;          												// correct border width
  fHeight -= 4;

																											// calculate tix in window
	float tmpTicksInWindow = (( maxOffset20 - minOffset) / 99.00) * (tmpZoomFactorTime-1) + minOffset;
  fTicksPerPix = tmpTicksInWindow / fWindowWidth;			// get tixperpix
	fWidth = (int)(maxOffset / fTicksPerPix);

	long tmpWidth = FindWidget( "drwControllerLabel" )->allocation.width;

	gtk_drawing_area_size( GTK_DRAWING_AREA( drwControllerMain ), fWidth, fHeight );
	gtk_drawing_area_size( GTK_DRAWING_AREA( FindWidget( "drwControllerLabel" )), tmpWidth, fHeight );

	fXAdjustment->upper = fWidth;

	if ( fBufferMain != 0 ) { gdk_pixmap_unref( fBufferMain ); }
	if ( fBufferLabel != 0 ) { gdk_pixmap_unref( fBufferLabel ); }

	fBufferMain = gdk_pixmap_new( drwControllerMain->window, fWidth, fHeight, -1 );
	fBufferLabel = gdk_pixmap_new( FindWidget( "drwControllerLabel" )->window, tmpWidth, fHeight, - 1 );

	if ( fHeader != 0 )	                                // resize header
	{	fHeader->Resize( fWidth, fTicksPerPix, fWindowWidth, fMeasureCount + cCountMeasureTail ); }
}

//=============================================================================
// redraw area
// 22.02.2001 Adrian Meyer
//=============================================================================
bool TMvcController::ControllerExpose( GtkWidget* aWidget, GdkEventExpose* aEvent )
{
	GdkGC* tmpGC = gdk_gc_new( aWidget->window );				// new graphic context
	gdk_gc_copy( tmpGC, aWidget->style->fg_gc[aWidget->state] );

	GdkDrawable* tmpWnd = aWidget->window;							// window to draw to

	long tmpFromX = (int)fXAdjustment->value;						// draw from (top left corner)					
	fWidth = aWidget->allocation.width - 1;							// width
	fHeight = aWidget->allocation.height - 1;						// height

	gdk_gc_set_foreground( tmpGC, colBackground );			// draw background
	gdk_draw_rectangle( fBufferMain, tmpGC, TRUE,	0, 0, fWidth, fHeight );

	gdk_gc_set_foreground( tmpGC, colGrid );						// draw grid
	gdk_gc_set_line_attributes( tmpGC, 0, GDK_LINE_SOLID, GDK_CAP_NOT_LAST, GDK_JOIN_MITER );

//  for ( int i = 0; i<fSequence->GetTrackCount(); i++ )
//		{	gdk_draw_line( fBufferController, tmpGC, 1, i*16, fWidth, i*16 ); }

                                                      // measure grid
	for ( long i=1; i<=fMeasureCount + cCountMeasureTail; i++ )
	{
		long tmpBeatCount;
		long tmpTicksPerBeat;
		long x = (long)(fSequence->GetOffset( i, 1, 0 ) / fTicksPerPix);

		fSequence->GetTimeInformation( i, &tmpBeatCount, &tmpTicksPerBeat );

		for ( long j=0; j<tmpBeatCount; j++ )
		{
			long tmpx = (long)(x + (( j*tmpTicksPerBeat ) / fTicksPerPix ));
			
			if ( tmpx >= tmpFromX && tmpx <= ( tmpFromX+fWindowWidth ) )
			{
  			gdk_gc_set_foreground( tmpGC, colSubGrid );		// draw beat lines
  			gdk_draw_line( fBufferMain, tmpGC, tmpx-tmpFromX, 1, tmpx-tmpFromX, fHeight );
			}
		}

		if ( x >= tmpFromX && x <= ( tmpFromX+fWindowWidth ) )
		{
  		gdk_gc_set_foreground( tmpGC, colGrid );				// draw measure lines
  		gdk_draw_line( fBufferMain, tmpGC, x-tmpFromX, 1, x-tmpFromX, fHeight );
		}
	}

/*	gdk_gc_set_line_attributes( tmpGC, 1, GDK_LINE_SOLID, GDK_CAP_NOT_LAST, GDK_JOIN_MITER );
	gdk_gc_set_foreground( tmpGC, colMidi );

	if ( fSequence->GetTrackCount() > 0 )
	{
		for ( long i=1; i<fSequence->GetTrackCount(); i++ )
		{
			if ( fSequence->GetTrack( i )->GetEvent(0) != 0 )
			{
				TMidiEvent* tmpEvent = fSequence->GetTrack( i )->GetEvent(0);
				while (	tmpEvent != 0 )
				{
					long cSpace = 2;
					long tmpx1 = (long)(tmpEvent->GetOffset() / fTicksPerPix);
					long tmpx2 = cSpace;

					if (((( tmpEvent->GetData1()) & 0xF0 ) == cESNoteOn ) & !(tmpEvent->GetNoteOff() ))
					{
						TMidiEvent* tmpEventRef = tmpEvent->Ref;
						if ( tmpEventRef != 0 )
							{ tmpx2 = (long)(tmpEventRef->GetOffset() / fTicksPerPix) - tmpx1;	};

						if ( tmpx1 >= tmpFromX && tmpx1 <= tmpToX )
						{
							gdk_draw_rectangle( fBufferController, tmpGC, TRUE,	
								tmpx1, (i-1)*16 + cSpace , tmpx2 - cSpace , 16 - 2 * cSpace );
						}
					}
		
					tmpEvent = tmpEvent->Next;
				}
			}			
		}
	}*/

	gdk_gc_set_function( tmpGC, GDK_XOR );
	gdk_gc_set_foreground( tmpGC, colSelection );				// draw selection
	gdk_draw_rectangle( fBufferMain, tmpGC, TRUE,	
		(int)( fSequence->GetSelectionStart() / fTicksPerPix ), 1,
		(int)(( fSequence->GetSelectionEnd() / fTicksPerPix ) -
			( fSequence->GetSelectionStart() / fTicksPerPix )), fHeight );	

	gdk_gc_set_function( tmpGC, GDK_COPY );
	gdk_gc_set_clip_rectangle( tmpGC, &aEvent->area );	// drawable selection
	gdk_draw_pixmap( tmpWnd, tmpGC, fBufferMain, 0, 0, 0, 0, fWidth, fHeight );
	gdk_gc_set_clip_rectangle( tmpGC,	NULL );           // go!
	gdk_gc_destroy( tmpGC );

	return TRUE;
}

//=============================================================================
// scroll piano roll
// 14.09.2000 Adrian Meyer
//=============================================================================
void TMvcController::Scroll( void )
{
	gtk_widget_queue_draw( drwControllerMain );
	gtk_widget_queue_draw( FindWidget( "drwControllerLabel" ));
}

//=============================================================================
// mouse down in main view
// 22.02.2001 Adrian Meyer
//=============================================================================
void TMvcController::MouseDown( long x )
{
	fSelectionActive = true;
	fSequence->SetSelectionStart( GetOffsetOnGrid( x * fTicksPerPix, true ));
}

//=============================================================================
// mouse up in main view
// 22.02.2001 Adrian Meyer
//=============================================================================
void TMvcController::MouseUp( long x )
{
	fSelectionActive = false;
	fSequence->SetSelectionEnd( GetOffsetOnGrid( x * fTicksPerPix, false ));
}

//=============================================================================
// mouse move in main view
// 22.02.2001 Adrian Meyer
//=============================================================================
void TMvcController::MouseMove( long x )
{
	long tmpFromX = (int)fXAdjustment->value;											

	if ( fSelectionActive )
	{                                                   // end position on grid																											
		int tmpSelectionEnd = GetOffsetOnGrid( ( x + tmpFromX ) * fTicksPerPix, false );						
		if ( tmpSelectionEnd > fWidth * fTicksPerPix )		// more than max?
			{	tmpSelectionEnd = (int)(fWidth * fTicksPerPix); }	// correct it!

		if ( tmpSelectionEnd >= fSequence->GetSelectionStart() )
		{
			fSequence->SetSelectionEnd( tmpSelectionEnd );	// set selection end of sequence
		}
																											// scroll window?	
																											// in direction x ->
		if ( fXAdjustment->value < ( x - fWindowWidth + tmpFromX ) && fXAdjustment->value < fWidth - fWindowWidth )
		{
																											// increase value by 10th of mouse position
			fXAdjustment->value += ( (x + tmpFromX) - ( fXAdjustment->value + fWindowWidth )) / 5 ;
			if ( fXAdjustment->value > fWidth - fWindowWidth )// if more than max
				{ fXAdjustment->value = fWidth - fWindowWidth; }// correct to max
			gtk_adjustment_value_changed( fXAdjustment );		// set new position in x adjustment
		}
																											// in direction x <
		if ( fXAdjustment->value > ( x + tmpFromX ) && fXAdjustment->value > 0 )
		{
			fXAdjustment->value -= (fXAdjustment->value - x + tmpFromX ) / 5;

			if ( fXAdjustment->value < 0 )									// if less than 0
				{ fXAdjustment->value = 0; }									// correct to 0
			gtk_adjustment_value_changed( fXAdjustment );		// set new position in x adjustment
		}
	}
}

//=============================================================================
// read offset on grid
// 22.02.2001 Adrian Meyer
//=============================================================================
long TMvcController::GetOffsetOnGrid( long aOffset, bool aRoundDown )
{
	long tmpMeasure; long tmpBeat; long tmpTick;
	long tmpBeatCount; long tmpTicksPerBeat;

	if (gtk_toggle_button_get_active( (GtkToggleButton*)FindWidget( "rboControllerMeasure" )))
	{
		fSequence->GetMeasureBeatTick( aOffset, &tmpMeasure, &tmpBeat, &tmpTick );
		if ( aRoundDown )																	// round down
		{	return fSequence->GetOffset( tmpMeasure, 1, 0 ); }
		else																							// round up
		{	return fSequence->GetOffset( tmpMeasure + 1, 1, 0 ) - 1; }
	}	

	if (gtk_toggle_button_get_active( (GtkToggleButton*)FindWidget( "rboControllerBeat" )))
	{
		fSequence->GetMeasureBeatTick( aOffset, &tmpMeasure, &tmpBeat, &tmpTick );
		if ( aRoundDown )																	// round down
		{
  		fSequence->GetMeasureBeatTick( aOffset, &tmpMeasure, &tmpBeat, &tmpTick );
  		fSequence->GetTimeInformation( tmpMeasure, &tmpBeatCount, &tmpTicksPerBeat );
			return fSequence->GetOffset( tmpMeasure, tmpBeat, 0 );
		}
		else
		{
  		fSequence->GetMeasureBeatTick( aOffset, &tmpMeasure, &tmpBeat, &tmpTick );
  		fSequence->GetTimeInformation( tmpMeasure, &tmpBeatCount, &tmpTicksPerBeat );
			return fSequence->GetOffset( tmpMeasure, tmpBeat, tmpTicksPerBeat - 1 );
		}
	}	

	return aOffset;
}

//=============================================================================
// time zoom changed
// 22.02.2001 Adrian Meyer
//=============================================================================
void ControllerTimeZoomChanged( GtkAdjustment *adjustment, gpointer user_data )
{
	TMvcController* tmpMvcController = (TMvcController*)user_data;
  tmpMvcController->Resize();
}

//=============================================================================
// redraw area
// 22.02.2001 Adrian Meyer
//=============================================================================
bool ControllerExposeHook( GtkWidget *widget, GdkEventExpose *event, gpointer user_data )
{
	TMvcController* tmpMvcController = (TMvcController*)user_data;
  return tmpMvcController->ControllerExpose( widget, event );
}

//=============================================================================
// size change of area
// 22.02.2001 Adrian Meyer
//=============================================================================
void ControllerSizeRequest( GtkWidget *widget, GtkRequisition *requisition, gpointer user_data )
{
	TMvcController* tmpMvcController = (TMvcController*)user_data;
  tmpMvcController->Resize();
}

//=============================================================================
// scrolling window
// 14.09.2000 Adrian Meyer
//=============================================================================
void ControllerScroll( GtkAdjustment *adjustment, gpointer user_data )
{
	TMvcController* tmpMvcController = (TMvcController*)user_data;
	tmpMvcController->Scroll();
}

//=============================================================================
// mouse down in main view
// 22.02.2001 Adrian Meyer
//=============================================================================
bool ControllerMouseDown( GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
	TMvcController* tmpMvcController = (TMvcController*)user_data;
	tmpMvcController->MouseDown( event->x );
	return true;
}

//=============================================================================
// mouse up in main view
// 22.02.2001 Adrian Meyer
//=============================================================================
bool ControllerMouseUp( GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
	TMvcController* tmpMvcController = (TMvcController*)user_data;
	tmpMvcController->MouseUp( event->x );
	return true;
}

//=============================================================================
// mouse move in main view
// 22.02.2001 Adrian Meyer
//=============================================================================
bool ControllerMouseMove( GtkWidget *widget, GdkEventMotion *event, gpointer user_data)
{
	TMvcController* tmpMvcController = (TMvcController*)user_data;
	tmpMvcController->MouseMove( event->x );
	return true;
}
